' Floating point test program
'
' Defined for the LAB-X1 using 16F877.  If need to use different LCD connections or a
' different chip, you must change the Define statements.  If your target device doesn't
' have ADC, remove the ADCON1=7 statement.
'
' Include only one of the following. Note that the 32 bit routines are not listed
' because they will not yield signed results.

	include "fp20.bas"	' Include file for 14-bit core with RAM at $20 (24-bit)
'	include "fp40.bas"	' Include file for 14-bit core with RAM at $40 (24-bit)
'	include "fp17.bas"	' Include file for 17Cxxx (24-bit)
'	include "fp18.bas"	' Include file for 18Xxxx (24-bit)

' Define LCD registers and bits
Define  LCD_DREG        PORTD
Define  LCD_DBIT        4
Define  LCD_RSREG       PORTE
Define  LCD_RSBIT       0
Define  LCD_EREG        PORTE
Define  LCD_EBIT        1

' Define the target device for PM assembler
Define  __16F877        1       ' Only use for PM, MPASM will define this by itself

fpplaces  var  byte			' Used to define number of decimal places in fpdisplay,fpdisplayr
ahold    var   word			' Required by fpdisplay
bhold    var   word			' Required by fpdisplay
' spin	 var   PORTC.6		' Use for serial output pin
' sbaud  var   396			' Use for serial output mode

resulthold	var	word		' Used to store results of previous calculations

		resulthold = 22		' This becomes the intitial value of aint


		ADCON1 = 7      	' Set PORTA and PORTE to digital
		Low PORTE.2     	' LCD R/W line low (W)
		Pause 500       	' Wait for LCD to start



' Demonstrate floating point addition.

loop:	aint = resulthold
		Gosub itofa			' Convert aint to float

		bint = 14
		Gosub itofb			' Convert bint to float

		Gosub fpadd			' FP add (+14)

		Lcdout $fe,1, sdec resulthold, " + 14 = "	' Clear display, display equation
		fpplaces = 2 : Gosub fpdisplayr				' Call display routine

'		Serout2 spin,sbaud,[sdec resulthold, " + 14 = ", sdec aint]	' Use for 2400 bps serial output

		Pause 3000



' Demonstrate floating point subtraction

		resulthold = aint	' Hold for later
		Gosub itofa			' Convert aint to float - aint=result from add

		bint = 17
		Gosub itofb			' Convert bint to float

		Gosub fpsub			' FP subtract (-17)

		Lcdout $fe, 1, sdec resulthold, " - 17 = "	' Clear display, display equation
		fpplaces = 2 : Gosub fpdisplayr				' Call display routine

'		Serout2 spin,sbaud,[sdec resulthold, " - 17 = ", sdec aint]	' Use for 2400 bps serial output
		Pause 3000



' Demonstrate floating point multiply

		resulthold = aint
        Gosub itofa     	' Convert aint to float  aint=result from subtract

        bint = 6
        Gosub itofb     	' Convert int to float

        Gosub fpmul     	' FP multiply (*6)

        Lcdout $fe, 1, sdec resulthold, " * 6 = "	' Clear display, display equation
		fpplaces = 2 : Gosub fpdisplayr 			' Call display routine

'		Serout2 spin,sbaud,[sdec resulthold, " * 6 = ", sdec aint]	' Use for 2400 bps serial output
        Pause 3000



' Demonstrate floating point divide

		resulthold = aint
        Gosub itofa     	' Convert int back to float - aint=result from multiply

        bint = 7
        Gosub itofb     	' Convert int to float

        Gosub fpdiv     	' FP divide (/7)

        Lcdout $fe, 1, sdec resulthold, " / 7 = "		' Clear display and output equation
'		Serout2 spin,sbaud,[sdec resulthold, " / 7 = "]	' Use for 2400 bps serial output

		fpplaces = 2 : Gosub fpdisplayr					' Call display routine

		Pause 3000

		resulthold = aint	' Store integer portion of result for next loop
        Goto loop			' Do it forever



' The fpdisplayr routine checks for number of places, then adds the appropriate
' value to achieve rounding.  To save RAM space, the values are hard coded
' in floating point format.

fpdisplayr: If fpplaces=0 Then			' Set floating point barg to 0.5
				bexp = $7E
				bargb0 = $00
				bargb1 = $01
			Endif
			If fpplaces=1 Then			' Set floating point barg to 0.05
				bexp = $7A
				bargb0 = $4C
				bargb1 = $CD
			Endif
			If fpplaces=2 Then			' Set floating point barg to 0.005
				bexp = $77
				bargb0 = $23
				bargb1 = $D8
			Endif
			If fpplaces=3 Then			' Set floating point barg to 0.0005
				bexp = $74
				bargb0 = $03
				bargb1 = $13
			Endif
			If fpplaces=4 Then			' Set floating point barg to 0.00005
				bexp = $70
				bargb0 = $51
				bargb1 = $B7
			Endif

			If aargb0.7 Then			' Check the floating point sign bit
				Gosub fpsub				' If negative, subtract barg from aarg
			Else
				Gosub fpadd				' If positive, add barg to aarg
			Endif


' The fpdisplay routine outputs the signed value of aarg in decimal floating point format. It
' can display an absolute value of 32767, and decimals to 4 places. The number of decimal
' places should be stored in fpplaces before calling the routine. The routine reads the 
' floating point value of aarg. This value should NOT be converted to an integer before 
' calling fpdisplay. The integer conversion will be perfomed as part of this routine, and
' aint will be returned to the calling program just as from the itofa routine.

fpdisplay:	bexp = aexp					' Store the FP value of aarg to the barg variables
			bargb0 = aargb0
			bargb1 = aargb1

			Gosub ftoia					' Convert aarg to integer
			ahold = aint				' Save this value for the final display
			
			Gosub itofa					' Convert integer back to float

			Swap aexp,bexp				' Swap the FP values of aarg and barg before subtraction
			Swap aargb0,bargb0
			Swap aargb1,bargb1

			Gosub fpsub					' Subtract the integer portion from the full number


			bint = 10					' Make bint = 10 E fpplaces
			If fpplaces=2 Then
				bint = 100
			Endif
			If fpplaces=3 Then
				bint = 1000
			Endif
			If fpplaces=4 Then
				bint = 10000
			Endif

			bhold = bint				' Save the integer value of bint for zeros loop

			Gosub itofb					' Convert bint to integer prior to FP multiply
			Gosub fpmul					' Multiply the decimal portion x 10 E fpplaces

			Gosub ftoia					' Convert result to aint integer


			If (ahold.15) OR (aint.15) Then		' Check for negative integer or decimal
				Lcdout "-"				' Display minus

'				Serout2 spin,sbaud,["-"]	' Use for 2400 bps serial output
			Endif

			Lcdout dec abs ahold		' Display integer portion

'			Serout2 spin,sbaud,[dec abs ahold]	' Use for 2400 bps serial output

			If fpplaces > 0 Then					
				Lcdout "."
'				Serout2 spin,sbaud,["."]	' Use for 2400 bps serial output

zeros:			bhold = bhold / 10								' Set bhold to be next place to right
				If (abs aint < bhold) AND (bhold > 1) Then		' Check for leading zero in decimal
					Lcdout "0"									' Display leading zero
'					Serout2 spin,sbaud,["0"]					' Use for 2400 bps serial output
					Goto zeros									' loop to check for another zero
				Endif
	
				Lcdout dec abs aint		' Display the rest of the decimal portion

'				Serout2 spin,sbaud,[dec abs aint]	' Use for 2400 bps serial output

			Endif

			aint = ahold				' Restore the original value of aint

			Return
